/*
	Copyright (C) 2004-2005 Cory Nelson

	This software is provided 'as-is', without any express or implied
	warranty.  In no event will the authors be held liable for any damages
	arising from the use of this software.

	Permission is granted to anyone to use this software for any purpose,
	including commercial applications, and to alter it and redistribute it
	freely, subject to the following restrictions:

	1. The origin of this software must not be misrepresented; you must not
		claim that you wrote the original software. If you use this software
		in a product, an acknowledgment in the product documentation would be
		appreciated but is not required.
	2. Altered source versions must be plainly marked as such, and must not be
		misrepresented as being the original software.
	3. This notice may not be removed or altered from any source distribution.
	
	CVS Info :
		$Author: phrostbyte $
		$Date: 2005/05/18 00:51:03 $
		$Revision: 1.4 $
*/

#include "stdafx.h"
using namespace std;

Driver::Driver() : dev(INVALID_HANDLE_VALUE),loaded(false),started(false),removable(false) {
}

Driver::~Driver() {
	this->Close();
}

void Driver::Load(const tstring &name) {
	TCHAR path[MAX_PATH+1];
	TCHAR *pathend;

	if(!SearchPath(NULL, name.c_str(), _T(".sys"), MAX_PATH, path, &pathend))
		throw driver_error("SearchPath");

	this->Load(name, path);
}

void Driver::Load(const tstring &name, const tstring &file) {
	this->Load(name, file, _T("\\\\.\\")+name);
}

void Driver::Load(const tstring &name, const tstring &file, const tstring &devfile) {
	if(this->loaded) throw runtime_error("Driver already loaded, close it first.");

	this->name=name;
	this->file=file;
	this->devfile=devfile;

	SC_HANDLE manager=OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
	if(!manager) throw driver_error("OpenSCManager");

	SC_HANDLE service=CreateService(manager, this->name.c_str(), this->name.c_str(),
		SERVICE_ALL_ACCESS, SERVICE_KERNEL_DRIVER, SERVICE_DEMAND_START, SERVICE_ERROR_NORMAL,
		this->file.c_str(), NULL, NULL, NULL, NULL, NULL);

	if(!service) {
		service=OpenService(manager, this->name.c_str(), SERVICE_ALL_ACCESS);

		if(!service) {
			CloseServiceHandle(manager);
			throw driver_error("CreateService/OpenService");
		}
	}

	SERVICE_STATUS status;
	if(QueryServiceStatus(service, &status))
		this->started=(status.dwCurrentState==SERVICE_RUNNING);

	CloseServiceHandle(service);
	CloseServiceHandle(manager);

	this->loaded=true;
}

void Driver::Close() {
	if(!this->loaded) return;
	this->Stop();

	if(this->removable) {
		SC_HANDLE manager=OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
		if(!manager) throw driver_error("OpenSCManager");

		SC_HANDLE service=OpenService(manager, this->name.c_str(), SERVICE_ALL_ACCESS);
		if(!service) {
			CloseServiceHandle(manager);
			throw driver_error("OpenService");
		}

		DeleteService(service);

		CloseServiceHandle(service);
		CloseServiceHandle(manager);
	}

	this->loaded=false;
}

void Driver::Start() {
	if(this->started) return;

	SC_HANDLE manager=OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
	if(!manager) throw driver_error("OpenSCManager");

	SC_HANDLE service=OpenService(manager, this->name.c_str(), SERVICE_ALL_ACCESS);
	if(!service) {
		CloseServiceHandle(manager);
		throw driver_error("OpenService");
	}

	if(!StartService(service, 0, NULL) && GetLastError()!=ERROR_SERVICE_ALREADY_RUNNING) {
		CloseServiceHandle(service);
		CloseServiceHandle(manager);

		throw driver_error("StartService");
	}

	CloseServiceHandle(service);
	CloseServiceHandle(manager);

	this->dev=CreateFile(this->devfile.c_str(), GENERIC_READ|GENERIC_WRITE,
		0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL|FILE_FLAG_OVERLAPPED, NULL);
	if(this->dev==NULL) throw driver_error("CreateFile");

	this->started=true;
}

void Driver::Stop() {
	if(!this->started) return;

	if(this->dev!=INVALID_HANDLE_VALUE)
		CloseHandle(this->dev);

	SC_HANDLE manager=OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);
	if(!manager) throw driver_error("OpenSCManager");

	SC_HANDLE service=OpenService(manager, this->name.c_str(), SERVICE_ALL_ACCESS);
	if(!service) {
		CloseServiceHandle(manager);
		throw driver_error("OpenService");
	}

	SERVICE_STATUS status;
	if(
		(
			!QueryServiceStatus(service, &status) ||
			(
				status.dwCurrentState!=SERVICE_STOPPED && 
				status.dwCurrentState!=SERVICE_STOP_PENDING
			)
		) &&
		!ControlService(service, SERVICE_CONTROL_STOP, &status)) {
			CloseServiceHandle(service);
			CloseServiceHandle(manager);
			throw driver_error("ControlService");
	}

	CloseServiceHandle(service);
	CloseServiceHandle(manager);

	this->started=false;
}

bool Driver::RawIo(DWORD ioctl, LPVOID inbuf, DWORD insize, LPVOID outbuf, DWORD outsize, LPOVERLAPPED overlapped) {
	DWORD ret;
	return (DeviceIoControl(this->dev, ioctl, inbuf, insize, outbuf, outsize, &ret, overlapped)!=0);
}

bool Driver::Read(DWORD ioctl, LPVOID buf, DWORD size, LPOVERLAPPED overlapped) {
	return this->RawIo(ioctl, NULL, 0, buf, size, overlapped);
}

bool Driver::Write(DWORD ioctl, LPVOID buf, DWORD size, LPOVERLAPPED overlapped) {
	return this->RawIo(ioctl, buf, size, NULL, 0, overlapped);
}
