/*
	Copyright (C) 2004-2005 Cory Nelson

	This software is provided 'as-is', without any express or implied
	warranty.  In no event will the authors be held liable for any damages
	arising from the use of this software.

	Permission is granted to anyone to use this software for any purpose,
	including commercial applications, and to alter it and redistribute it
	freely, subject to the following restrictions:

	1. The origin of this software must not be misrepresented; you must not
		claim that you wrote the original software. If you use this software
		in a product, an acknowledgment in the product documentation would be
		appreciated but is not required.
	2. Altered source versions must be plainly marked as such, and must not be
		misrepresented as being the original software.
	3. This notice may not be removed or altered from any source distribution.
	
	CVS Info :
		$Author: phrostbyte $
		$Date: 2005/07/07 19:12:29 $
		$Revision: 1.10 $
*/

#ifndef __MESSAGEBOX_H__
#define __MESSAGEBOX_H__

#include "stdafx.h"
#include "resource.h"
using namespace std;

int MessageBox(HWND hwnd, const tstring &text, const tstring &caption, UINT type) {
	return MessageBox(hwnd, text.c_str(), caption.c_str(), type);
}

int MessageBox(HWND hwnd, const tstring &text, UINT captionid, UINT type) {
	return MessageBox(hwnd, text, LoadString(captionid), type);
}

int MessageBox(HWND hwnd, UINT textid, UINT captionid, UINT type) {
	return MessageBox(hwnd, LoadString(textid), LoadString(captionid), type);
}

#ifndef PG2_REPORT_BUGS

#define ReportException(x,y,z)

#else

static const char *g_bughost="bugs.methlabs.org";
static const char *g_bugport="50005";

static void ReportException(const exception *ex, const char *file, int line) {
	string packet;
	{
		ostringstream buf;

		buf
			<< g_build
			<< '\n'
			<< file
			<< '\n'
			<< line;

		if(ex) {
			buf
				<< '\n'
				<< typeid(*ex).name()
				<< '\n'
				<< MBS_UTF8(ex->what());
		}

		packet=buf.str();
	}

	SOCKET sock=INVALID_SOCKET;
	{
		addrinfo hints={0};
		hints.ai_socktype=SOCK_DGRAM;
		hints.ai_protocol=IPPROTO_UDP;

		addrinfo *info;
		if(getaddrinfo(g_bughost, g_bugport, &hints, &info)!=0) return;

		for(const addrinfo *iter=info; iter!=NULL; iter=iter->ai_next) {
			sock=socket(iter->ai_family, iter->ai_socktype, iter->ai_protocol);
			if(sock==INVALID_SOCKET) continue;

			if(connect(sock, iter->ai_addr, (socklen_t)iter->ai_addrlen)!=SOCKET_ERROR) {
				break;
			}
			else {
				closesocket(sock);
				sock=INVALID_SOCKET;
			}
		}

		freeaddrinfo(info);
	}

	if(sock==INVALID_SOCKET) return;

	send(sock, packet.c_str(), (socklen_t)packet.size(), 0);
	closesocket(sock);
}

#endif

void ExceptionBox(HWND hwnd, const exception &ex, const char *file, int line) {
	ReportException(&ex, file, line);

	tstring str=boost::str(tformat(LoadString(IDS_EXCEPTIONTEXT))%g_build%file%line%typeid(ex).name()%ex.what());
	MessageBox(hwnd, str, IDS_EXCEPTION, MB_ICONERROR|MB_OK);
}

void UncaughtExceptionBox(HWND hwnd, const char *file, int line) {
	ReportException(NULL, file, line);

	tstring str=boost::str(tformat(LoadString(IDS_CAUGHTUNKNOWNTEXT))%g_build%file%line);
	MessageBox(hwnd, str, IDS_UNCAUGHT, MB_ICONERROR|MB_OK);
}

void UncaughtExceptionBox(HWND hwnd, const exception &ex, const char *file, int line) {
	ReportException(&ex, file, line);

	tstring str=boost::str(tformat(LoadString(IDS_UNCAUGHTTEXT))%g_build%file%line%typeid(ex).name()%ex.what());
	MessageBox(hwnd, str, IDS_UNCAUGHT, MB_ICONERROR|MB_OK);
}