/*
	Copyright (C) 2004-2005 Cory Nelson

	This software is provided 'as-is', without any express or implied
	warranty.  In no event will the authors be held liable for any damages
	arising from the use of this software.

	Permission is granted to anyone to use this software for any purpose,
	including commercial applications, and to alter it and redistribute it
	freely, subject to the following restrictions:

	1. The origin of this software must not be misrepresented; you must not
		claim that you wrote the original software. If you use this software
		in a product, an acknowledgment in the product documentation would be
		appreciated but is not required.
	2. Altered source versions must be plainly marked as such, and must not be
		misrepresented as being the original software.
	3. This notice may not be removed or altered from any source distribution.
	
	CVS Info :
		$Author: phrostbyte $
		$Date: 2005/04/28 00:10:50 $
		$Revision: 1.9 $
*/

#include "stdafx.h"
using namespace std;

// Increasing loop_count will slightly lower cpu usage.
// A mutex will be re-locked every loop_count loops.
// SetRanges will block for up to around loop_count*sleep_ms milliseconds.
// ~PgFilter will block for up to around sleep_ms milliseconds.
static const boost::uint_fast16_t loop_count=20;
static const DWORD sleep_ms=25;

PgFilter::PgFilter() : block(false),blockhttp(true) {
	this->SetBlock(true);
}

PgFilter::~PgFilter() {
	this->SetBlock(false);
}

void PgFilter::notifythread() {
	while(this->block) {
		mutex::scoped_lock lock(this->mutex);

		for(boost::uint_fast16_t i=0; this->block && i<loop_count; i++) {
			MIB_TCPTABLE *table=NULL;
			DWORD tablelen=0;

			GetTcpTable(table, &tablelen, FALSE);

			table=(MIB_TCPTABLE*)malloc(tablelen);
			if(GetTcpTable(table, &tablelen, FALSE)!=NO_ERROR) {
				free(table);
				continue;
			}

			for(DWORD i=0; this->block && i<table->dwNumEntries; i++) {
				unsigned short remoteport=(unsigned short)ntohs((unsigned short)table->table[i].dwRemotePort);
				if(!this->blockhttp && (remoteport==80 || remoteport==443)) continue;

				unsigned int remote=ntohl(table->table[i].dwRemoteAddr);

				const Range *r=this->inranges(remote);

				if(r) {
					unsigned short localport=(unsigned short)ntohs((unsigned short)table->table[i].dwLocalPort);
					unsigned int local=ntohl(table->table[i].dwLocalAddr);

					table->table[i].dwState=MIB_TCP_STATE_DELETE_TCB;

					if(SetTcpEntry(&table->table[i])==NO_ERROR && !this->OnAction.empty()) {
						Action a;
						a.Name=r->Name;
						a.SourceIp=local;
						a.SourcePort=localport;
						a.DestIp=remote;
						a.DestPort=remoteport;
						a.Protocol=IPPROTO_TCP;
						a.Blocked=true;

						try {
							this->OnAction(a);
						}
						catch(exception &ex) {
							ExceptionBox(NULL, ex, __FILE__, __LINE__);
							// try to stay alive
						}
					}
				}
			}

			free(table);
			Sleep(sleep_ms);
		}
	}
}

const PgFilter::Range *PgFilter::inranges(unsigned int ip) const {
	int high=(int)ranges.size(), low=-1, probe;

	while(high-low > 1) {
		probe=(high+low)/2;
		if(ranges[probe].Start>ip)
			high=probe;
		else low=probe;
	}

	if(low==-1 || ip<ranges[low].Start || ip>ranges[low].End) return NULL;
	else return &ranges[low];
}

void PgFilter::SetRanges(const p2p::list &ranges) {
	mutex::scoped_lock lock(this->mutex);
	
	this->ranges.clear();

	map<wstring,LPCTSTR> names;
	for(p2p::list::const_iterator iter=ranges.begin(); iter!=ranges.end(); iter++)
		names[iter->name]=NULL;

	size_t nameslen=names.size();
	for(map<wstring,LPCTSTR>::const_iterator iter=names.begin(); iter!=names.end(); iter++)
		nameslen+=WCHAR_TSTRING(iter->first).length();

	this->names=boost::shared_array<TCHAR>(new TCHAR[nameslen]);
	TCHAR *bufiter=this->names.get();

	for(map<wstring,LPCTSTR>::iterator iter=names.begin(); iter!=names.end(); iter++) {
		tstring str=WCHAR_TSTRING(iter->first);
		LPCTSTR ptr=str.c_str();
		size_t len=str.length()+1;
		copy(ptr, ptr+len, bufiter);

		iter->second=bufiter;
		bufiter+=len;
	}

	this->blockedcount=(unsigned int)ranges.size();

	for(p2p::list::const_iterator iter=ranges.begin(); iter!=ranges.end(); iter++) {
		Range r;

		r.Name=names[iter->name];
		r.Start=iter->start.ipl;
		r.End=iter->end.ipl;

		this->ranges.push_back(r);
		this->blockedcount+=r.End - r.Start;
	}
}

void PgFilter::SetBlock(bool block) {
	this->block=block;

	if(block && !this->thread.get())
		this->thread=boost::shared_ptr<::thread>(new ::thread(boost::bind(&PgFilter::notifythread, this)));
	else if(!block && this->thread.get()) {
		this->thread->join();
		this->thread=boost::shared_ptr<::thread>();
	}
}

void PgFilter::SetBlockHttp(bool block) {
	this->blockhttp=block;
}
