/*
	Copyright (C) 2004-2005 Cory Nelson

	This software is provided 'as-is', without any express or implied
	warranty.  In no event will the authors be held liable for any damages
	arising from the use of this software.

	Permission is granted to anyone to use this software for any purpose,
	including commercial applications, and to alter it and redistribute it
	freely, subject to the following restrictions:

	1. The origin of this software must not be misrepresented; you must not
		claim that you wrote the original software. If you use this software
		in a product, an acknowledgment in the product documentation would be
		appreciated but is not required.
	2. Altered source versions must be plainly marked as such, and must not be
		misrepresented as being the original software.
	3. This notice may not be removed or altered from any source distribution.
	
	CVS Info :
		$Author: phrostbyte $
		$Date: 2005/04/09 14:14:41 $
		$Revision: 1.6 $
*/

#ifndef __PGFILTER_H__
#define __PGFILTER_H__

#include <vector>
#include <boost/function.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/shared_array.hpp>
#include <boost/utility.hpp>
#include <p2p/range.hpp>
#include <windows.h>
#include "driver.h"
#include "tstring.h"
#include "threadx.hpp"

class PgFilter : boost::noncopyable {
public:
	struct Action {
		tstring Name;
		unsigned int SourceIp, DestIp;
		unsigned short SourcePort, DestPort;
		int Protocol;
		bool Blocked;
	};

private:
	Driver ipfltdrv;
	Driver filter;

	boost::shared_ptr<thread> thread;
	mutex mutex;

	boost::shared_array<TCHAR> ranges;
	unsigned int blockedcount;

	bool block, blockhttp;
	bool runthread;

	void notifythread();
	bool isrunning();

public:
	boost::function<void(const Action&)> OnAction;

	PgFilter();
	~PgFilter();

	void SetRanges(const p2p::list &ranges);

	void SetBlock(bool block);
	void SetBlockHttp(bool block);

	bool Hooked() const { return this->block; }
	bool BlockingHttp() const { return this->blockhttp; }
	unsigned int BlockedCount() const { return this->blockedcount; }
};

extern boost::shared_ptr<PgFilter> filter;

#endif
