/*
	Copyright (C) 2004-2005 Cory Nelson
	Based on the original work by Tim Leonard

	This software is provided 'as-is', without any express or implied
	warranty.  In no event will the authors be held liable for any damages
	arising from the use of this software.

	Permission is granted to anyone to use this software for any purpose,
	including commercial applications, and to alter it and redistribute it
	freely, subject to the following restrictions:

	1. The origin of this software must not be misrepresented; you must not
		claim that you wrote the original software. If you use this software
		in a product, an acknowledgment in the product documentation would be
		appreciated but is not required.
	2. Altered source versions must be plainly marked as such, and must not be
		misrepresented as being the original software.
	3. This notice may not be removed or altered from any source distribution.
	
	CVS Info :
		$Author: phrostbyte $
		$Date: 2005/02/26 05:31:35 $
		$Revision: 1.2 $
*/

#include <ntddk.h>
#include "filter.h"
#include "notificationqueue.h"

typedef struct __pg_notifynode {
	LIST_ENTRY entry;
	PGNOTIFICATION notification;
} PGNOTIFYNODE;

typedef struct __pg_irpnode {
	LIST_ENTRY entry;
	PIRP irp;
} PGIRPNODE;

void InitNotificationQueue(NOTIFICATION_QUEUE *queue) {
	InitializeListHead(&queue->irp_list);
	InitializeListHead(&queue->notification_list);
	KeInitializeSpinLock(&queue->lock);
}

void DestroyNotificationQueue(NOTIFICATION_QUEUE *queue) {
	KIRQL irq;

	KeAcquireSpinLock(&queue->lock, &irq);

	while(!IsListEmpty(&queue->notification_list)) {
		PGNOTIFYNODE *notifynode=(PGNOTIFYNODE*)RemoveHeadList(&queue->notification_list);
		ExFreePool(notifynode);
	}

	while(!IsListEmpty(&queue->irp_list)) {
		PGIRPNODE *irpnode=(PGIRPNODE*)RemoveHeadList(&queue->irp_list);
		
		KeReleaseSpinLock(&queue->lock, irq);
		
		irpnode->irp->IoStatus.Status=STATUS_CANCELLED;
		irpnode->irp->IoStatus.Information=0;

		IoCompleteRequest(irpnode->irp, IO_NO_INCREMENT);

		ExFreePool(irpnode);
		
		KeAcquireSpinLock(&queue->lock, &irq);
	}
	
	KeReleaseSpinLock(&queue->lock, irq);
}

void Notification_Send(NOTIFICATION_QUEUE *queue, const PGNOTIFICATION *notification) {
	KIRQL irq;

	KeAcquireSpinLock(&queue->lock, &irq);

	if(IsListEmpty(&queue->irp_list)) {
		PGNOTIFYNODE *notifynode;

		notifynode=ExAllocatePool(NonPagedPool, sizeof(PGNOTIFYNODE));
		
		InitializeListHead(&notifynode->entry);
		RtlCopyMemory(&notifynode->notification, notification, sizeof(PGNOTIFICATION));

		InsertTailList(&queue->notification_list, &notifynode->entry);

		KeReleaseSpinLock(&queue->lock, irq);
	}
	else {
		PGIRPNODE *irpnode=(PGIRPNODE*)RemoveHeadList(&queue->irp_list);
		PGNOTIFICATION *irpnotification=irpnode->irp->AssociatedIrp.SystemBuffer;

		KeReleaseSpinLock(&queue->lock, irq);

		RtlCopyMemory(irpnotification, notification, sizeof(PGNOTIFICATION));

		irpnode->irp->IoStatus.Status=STATUS_SUCCESS;
		irpnode->irp->IoStatus.Information=sizeof(PGNOTIFICATION);

		IoCompleteRequest(irpnode->irp, IO_NO_INCREMENT);

		ExFreePool(irpnode);
	}
}

NTSTATUS Notification_Recieve(NOTIFICATION_QUEUE *queue, PIRP irp) {
	PIO_STACK_LOCATION irpstack=IoGetCurrentIrpStackLocation(irp);
	KIRQL irq;

	if(irpstack->Parameters.DeviceIoControl.OutputBufferLength!=sizeof(PGNOTIFICATION)) {
		irp->IoStatus.Status=STATUS_BUFFER_TOO_SMALL;
		irp->IoStatus.Information=0;
		IoCompleteRequest(irp, IO_NO_INCREMENT);

		return STATUS_BUFFER_TOO_SMALL;
	}
	
	KeAcquireSpinLock(&queue->lock, &irq);

	if(IsListEmpty(&queue->notification_list)) {
		PGIRPNODE *irpnode;

		irpnode=ExAllocatePool(NonPagedPool, sizeof(PGIRPNODE));

		InitializeListHead(&irpnode->entry);
		irpnode->irp=irp;

		InsertTailList(&queue->irp_list, &irpnode->entry);
		
		IoMarkIrpPending(irp);		

		KeReleaseSpinLock(&queue->lock, irq);
		return STATUS_PENDING;
	}
	else {
		PGNOTIFYNODE *notifynode=(PGNOTIFYNODE*)RemoveHeadList(&queue->notification_list);
		PGNOTIFICATION *notification=irp->AssociatedIrp.SystemBuffer;

		KeReleaseSpinLock(&queue->lock, irq);

		RtlCopyMemory(notification, &notifynode->notification, sizeof(PGNOTIFICATION));
		ExFreePool(notifynode);

		irp->IoStatus.Status=STATUS_SUCCESS;
		irp->IoStatus.Information=sizeof(PGNOTIFICATION);
		IoCompleteRequest(irp, IO_NO_INCREMENT);

		return STATUS_SUCCESS;
	}
}
