/*
	Copyright (C) 2004-2005 Cory Nelson

	This software is provided 'as-is', without any express or implied
	warranty.  In no event will the authors be held liable for any damages
	arising from the use of this software.

	Permission is granted to anyone to use this software for any purpose,
	including commercial applications, and to alter it and redistribute it
	freely, subject to the following restrictions:

	1. The origin of this software must not be misrepresented; you must not
		claim that you wrote the original software. If you use this software
		in a product, an acknowledgment in the product documentation would be
		appreciated but is not required.
	2. Altered source versions must be plainly marked as such, and must not be
		misrepresented as being the original software.
	3. This notice may not be removed or altered from any source distribution.
	
	CVS Info :
		$Author: phrostbyte $
		$Date: 2005/04/23 22:51:48 $
		$Revision: 1.3 $
*/

#include <tchar.h>
#include <stdio.h>
#include <process.h>
#include <windows.h>
#include <psapi.h>
#include "resource.h"

static int MessageBoxString(HWND hwnd, UINT caption, UINT text, UINT type) {
	TCHAR c[64], t[512];

	LoadString(GetModuleHandle(NULL), caption, c, 64);
	LoadString(GetModuleHandle(NULL), text, t, 512);

	return MessageBox(hwnd, t, c, type);
}

static void AddListBoxString(HWND list, UINT text) {
	TCHAR t[512];
	LoadString(GetModuleHandle(NULL), text, t, 512);

	SendMessage(list, LB_ADDSTRING, 0, (LPARAM)t);
}

static void fixthread(void *arg) {
	HWND hwnd=(HWND)arg;
	HWND list=GetDlgItem(hwnd, IDC_LIST);
	HWND close=GetDlgItem(hwnd, IDCLOSE);

	SC_HANDLE manager=OpenSCManager(NULL, NULL, SC_MANAGER_ALL_ACCESS);

	DWORD *procids, ret, proclen, modlen, i, j;
	HMODULE *modules;

	if(manager) {
		SC_HANDLE service=OpenService(manager, _T("pgfilter"), SERVICE_ALL_ACCESS);

		if(service) {
			SERVICE_STATUS status;

			if(QueryServiceStatus(service, &status)) {
				if(status.dwCurrentState!=SERVICE_STOPPED) {
					AddListBoxString(list, IDS_STOPPING);

					if(!ControlService(service, SERVICE_CONTROL_STOP, &status)) {
						CloseServiceHandle(service);
						CloseServiceHandle(manager);
						
						AddListBoxString(list, IDS_CONTROLSERVICE);
						EnableWindow(close, TRUE);
						return;
					}
					Sleep(1000);

					AddListBoxString(list, IDS_STOPPED);
				}

				AddListBoxString(list, IDS_DELETING);
				if(DeleteService(service))
					AddListBoxString(list, IDS_DELETED);
				else if(GetLastError()==ERROR_SERVICE_MARKED_FOR_DELETE)
					AddListBoxString(list, IDS_MARKED);
				else AddListBoxString(list, IDS_DELETESERVICE);
			}
			else AddListBoxString(list, IDS_QUERYSERVICESTATUS);
			
			CloseServiceHandle(service);
		}
		else AddListBoxString(list, IDS_NOTFOUND);

		CloseServiceHandle(manager);
	}
	else AddListBoxString(list, IDS_OPENSCMANAGER);

	proclen=sizeof(DWORD)*128;
	procids=malloc(proclen);

	while(EnumProcesses(procids, proclen, &ret) && ret==proclen) {
		proclen*=2;
		procids=realloc(procids, proclen);
	}

	proclen=ret/sizeof(DWORD);

	modlen=sizeof(HMODULE)*128;
	modules=malloc(modlen);

	for(i=0; i<proclen; i++) {
		HANDLE proc=OpenProcess(PROCESS_QUERY_INFORMATION|PROCESS_VM_READ|PROCESS_TERMINATE, FALSE, procids[i]);

		if(proc!=NULL) {
			while(EnumProcessModules(proc, modules, modlen, &ret) && ret==modlen) {
				modlen*=2;
				modules=realloc(modules, modlen);
			}

			modlen=ret/sizeof(DWORD);

			for(j=0; j<modlen; j++) {
				TCHAR buf[MAX_PATH];
				GetModuleFileNameEx(proc, modules[j], buf, MAX_PATH);

				if(_tcsstr(buf, _T("pg2.exe"))) {
					if(TerminateProcess(proc, 0)) AddListBoxString(list, IDS_PROCTERMINATED);
					else AddListBoxString(list, IDS_TERMINATEPROCESS);
					break;
				}
			}
		}

		CloseHandle(proc);
	}

	free(modules);
	free(procids);

	EnableWindow(close, TRUE);
}

static INT_PTR CALLBACK PgFix_DlgProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam) {
	if(msg==WM_INITDIALOG) {
		HICON icon=(HICON)LoadImage(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_PGFIX), IMAGE_ICON, 0, 0, LR_SHARED|LR_DEFAULTSIZE);

		SendMessage(hwnd, WM_SETICON, ICON_BIG, (LPARAM)icon);
		SendMessage(hwnd, WM_SETICON, ICON_SMALL, (LPARAM)icon);

		if(MessageBoxString(hwnd, IDS_WARNING, IDS_WARNINGTEXT, MB_ICONWARNING|MB_YESNO)==IDYES)
			_beginthread(&fixthread, 0, (void*)hwnd);
		else EndDialog(hwnd, 0);

		return TRUE;
	}
	else if(msg==WM_COMMAND) {
		if(LOWORD(wParam)==IDCLOSE)
			EndDialog(hwnd, 0);
	}
	return FALSE;
}

static BOOL IsAdmin(void) {
	SID_IDENTIFIER_AUTHORITY auth=SECURITY_NT_AUTHORITY;
	PSID sid;
	BOOL b;

	if(!AllocateAndInitializeSid(&auth, 2, SECURITY_BUILTIN_DOMAIN_RID, DOMAIN_ALIAS_RID_ADMINS,
		0, 0, 0, 0, 0, 0, &sid)) return FALSE;

	if(!CheckTokenMembership(NULL, sid, &b))
		b=FALSE;

	FreeSid(sid);

	return b;
}

int WINAPI _tWinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow) {
	if(!IsAdmin()) {
		TCHAR text[512], caption[128];
		LoadString(GetModuleHandle(NULL), IDS_NEEDADMINTEXT, text, 512);
		LoadString(GetModuleHandle(NULL), IDS_NEEDADMIN, text, 128);

		MessageBox(NULL, text, caption, MB_ICONERROR|MB_OK);
		return -1;
	}

	DialogBox(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_PGFIX), NULL, PgFix_DlgProc);
	return 0;
}
